package com.imcode.forum.sql;

import com.imcode.forum.Forum;
import com.imcode.forum.ForumPost;
import com.imcode.forum.ForumThread;
import com.imcode.forum.SimpleForumPost;
import com.imcode.forum.Id;
import junit.framework.TestCase;
import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.beanutils.BeanUtils;

import java.util.SortedSet;
import java.util.Properties;
import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

public class SqlRepositoryTest extends TestCase {

    public void testMysql() throws Exception {
        SqlRepository repository = createRepository("mysql.test.properties");
        assertRepository(repository);
    }

    public void testMssql() throws Exception {
        SqlRepository repository = createRepository("mssql.test.properties");
        assertRepository(repository);
    }

    private SqlRepository createRepository(String propertiesFilename) throws Exception {
        BasicDataSource dataSource = createDataSource(propertiesFilename);
        SqlRepository repository = new SqlRepository(dataSource);
        repository.dropTables();
        return repository;
    }

    private BasicDataSource createDataSource(String propertiesFilename) throws Exception {
        Properties properties = new Properties();
        properties.load(new FileInputStream(propertiesFilename));
        BasicDataSource dataSource = new BasicDataSource();
        BeanUtils.populate(dataSource, properties);
        return dataSource;
    }

    private void assertRepository(SqlRepository repo) {
        Forum newForum = repo.createForum(repo.createId(new Integer(1)));
        Id forumId = newForum.getId() ;
        Forum forum = repo.getForum(forumId);
        assertNotNull(forum);

        String postTitle = "title";
        String postBody = "body";
        String postUser = "user";
        SimpleForumPost post = new SimpleForumPost(postTitle, postBody, postUser);
        ForumThread newThread = forum.createThread(post);
        ForumThread thread = forum.getThread(newThread.getId()) ;
        assertNotNull(thread);

        ForumPost topPost = thread.getTopPost();
        assertNotNull(topPost);
        assertEquals(postTitle, topPost.getTitle()) ;
        assertEquals(postBody, post.getBody());
        assertEquals(postUser, post.getUser());
        assertEquals(post.getDateTime().getTime() / 1000, topPost.getDateTime().getTime() / 1000) ;
        assertEquals(post.isHidden(), topPost.isHidden()) ;

        String replyTitle = "reply title";
        String replyBody = "reply body";
        String replyUser = "reply user";
        SimpleForumPost newReply = new SimpleForumPost(replyTitle, replyBody, replyUser);
        Id replyId = topPost.createReply(newReply);
        assertNotNull(replyId) ;
        ForumPost reply = thread.getPost(replyId) ;
        assertNotNull(reply) ;
        assertTrue(topPost.getReplies().contains(reply)) ;
        assertEquals(replyTitle, reply.getTitle()) ;
        assertEquals(replyBody, reply.getBody()) ;
        assertEquals(replyUser, reply.getUser()) ;

        Id replyReplyId = reply.createReply(newReply) ;
        ForumPost replyReply = thread.getPost(replyReplyId) ;
        assertNotNull(replyReply) ;
        assertTrue(reply.getReplies().contains(replyReply)) ;
        replyReply.hide(true);
        assertTrue(replyReply.isHidden()) ;
        assertTrue(thread.getPost(replyReplyId).isHidden()) ;

        replyReply.incrementViewCount();
        assertEquals(1, thread.getPost(replyReply.getId()).getViewCount());

        int postCount = thread.getPostCount();
        assertEquals(3, postCount) ;

        SortedSet<ForumPost> replies = topPost.getReplies();
        assertEquals(1, replies.size()) ;
        assertEquals(topPost.getId(), ( (SqlPost) replies.last() ).getParentPostId()) ;
    }

}
